@@ -18,6 +18,8 @@ const usage = `usage: sojuctl [-config path] <command>
 `
 
 func init() {
+	log.SetFlags(0)
+
 	flag.Usage = func() {
 		fmt.Fprintf(flag.CommandLine.Output(), usage)
 	}
@@ -33,9 +35,8 @@ func run(ctx context.Context, cfg *config.Server, words []string) error {
 		if u.Scheme != "unix+admin" {
 			continue
 		}
-		if u.Path != "" {
-			path = u.Path
-		} else {
+		path = u.Host + u.Path
+		if path == "" {
 			path = config.DefaultUnixAdminPath
 		}
 		break
