@@ -9,7 +9,7 @@ import (
 	"strings"
 	"time"
 
-	"git.sr.ht/~emersion/go-scfg"
+	"codeberg.org/emersion/go-scfg"
 )
 
 var (
@@ -81,7 +81,7 @@ type Server struct {
 
 	DB         DB
 	MsgStore   MsgStore
-	Auth       Auth
+	Auth       []Auth
 	FileUpload *FileUpload
 
 	HTTPOrigins    []string
@@ -106,11 +106,11 @@ func Defaults() *Server {
 			Source: "soju.db",
 		},
 		MsgStore: MsgStore{
-			Driver: "memory",
+			Driver: "db",
 		},
-		Auth: Auth{
+		Auth: []Auth{{
 			Driver: "internal",
-		},
+		}},
 		HTTPIngress:     "https://" + hostname,
 		MaxUserNetworks: -1,
 	}
@@ -121,22 +121,24 @@ func Load(filename string) (*Server, error) {
 		Listen []struct {
 			Addr string `scfg:",param"`
 		} `scfg:"listen"`
-		Hostname            string     `scfg:"hostname"`
-		Title               string     `scfg:"title"`
-		MOTD                string     `scfg:"motd"`
-		TLS                 *[2]string `scfg:"tls"`
-		DB                  *[2]string `scfg:"db"`
-		MessageStore        []string   `scfg:"message-store"`
-		Log                 []string   `scfg:"log"`
-		Auth                []string   `scfg:"auth"`
-		FileUpload          []string   `scfg:"file-upload"`
-		HTTPOrigin          []string   `scfg:"http-origin"`
-		HTTPIngress         string     `scfg:"http-ingress"`
-		AcceptProxyIP       []string   `scfg:"accept-proxy-ip"`
-		MaxUserNetworks     int        `scfg:"max-user-networks"`
-		UpstreamUserIP      []string   `scfg:"upstream-user-ip"`
-		DisableInactiveUser string     `scfg:"disable-inactive-user"`
-		EnableUserOnAuth    string     `scfg:"enable-user-on-auth"`
+		Hostname     string     `scfg:"hostname"`
+		Title        string     `scfg:"title"`
+		MOTD         string     `scfg:"motd"`
+		TLS          *[2]string `scfg:"tls"`
+		DB           *[2]string `scfg:"db"`
+		MessageStore []string   `scfg:"message-store"`
+		Log          []string   `scfg:"log"`
+		Auth         []struct {
+			Params []string `scfg:",param"`
+		} `scfg:"auth"`
+		FileUpload          []string `scfg:"file-upload"`
+		HTTPOrigin          []string `scfg:"http-origin"`
+		HTTPIngress         string   `scfg:"http-ingress"`
+		AcceptProxyIP       []string `scfg:"accept-proxy-ip"`
+		MaxUserNetworks     int      `scfg:"max-user-networks"`
+		UpstreamUserIP      []string `scfg:"upstream-user-ip"`
+		DisableInactiveUser string   `scfg:"disable-inactive-user"`
+		EnableUserOnAuth    string   `scfg:"enable-user-on-auth"`
 	}
 
 	raw.MaxUserNetworks = -1
@@ -177,7 +179,9 @@ func Load(filename string) (*Server, error) {
 		}
 		switch driver {
 		case "memory", "db":
-			// nothing to do
+			if source != "" {
+				return nil, fmt.Errorf("directive message-store: driver %q requires zero parameters", driver)
+			}
 		case "fs":
 			if source == "" {
 				return nil, fmt.Errorf("directive message-store: driver %q requires a source", driver)
@@ -187,22 +191,28 @@ func Load(filename string) (*Server, error) {
 		}
 		srv.MsgStore = MsgStore{driver, source}
 	}
-	if raw.Auth != nil {
-		driver, source, err := parseDriverSource("auth", raw.Auth)
+	if len(raw.Auth) != 0 {
+		// drop default auth if we are explicitly defining any auth
+		srv.Auth = nil
+	}
+	for _, auth := range raw.Auth {
+		driver, source, err := parseDriverSource("auth", auth.Params)
 		if err != nil {
 			return nil, err
 		}
 		switch driver {
 		case "internal", "pam":
-			// nothing to do
-		case "oauth2":
+			if source != "" {
+				return nil, fmt.Errorf("directive auth: driver %q requires zero parameters", driver)
+			}
+		case "http", "oauth2":
 			if source == "" {
 				return nil, fmt.Errorf("directive auth: driver %q requires a source", driver)
 			}
 		default:
 			return nil, fmt.Errorf("directive auth: unknown driver %q", driver)
 		}
-		srv.Auth = Auth{driver, source}
+		srv.Auth = append(srv.Auth, Auth{driver, source})
 	}
 	if raw.FileUpload != nil {
 		driver, source, err := parseDriverSource("file-upload", raw.FileUpload)
@@ -210,7 +220,7 @@ func Load(filename string) (*Server, error) {
 			return nil, err
 		}
 		switch driver {
-		case "fs":
+		case "fs", "http":
 			if source == "" {
 				return nil, fmt.Errorf("directive file-upload: driver %q requires a source", driver)
 			}
