@@ -10,11 +10,12 @@ import (
 	"strings"
 	"time"
 
-	"codeberg.org/emersion/soju/xirc"
 	_ "github.com/lib/pq"
 	"github.com/prometheus/client_golang/prometheus"
 	promcollectors "github.com/prometheus/client_golang/prometheus/collectors"
 	"gopkg.in/irc.v4"
+
+	"codeberg.org/emersion/soju/xirc"
 )
 
 const postgresQueryTimeout = 5 * time.Second
@@ -30,91 +31,6 @@ CREATE TABLE IF NOT EXISTS "Config" (
 //go:embed postgres_schema.sql
 var postgresSchema string
 
-var postgresMigrations = []string{
-	"", // migration #0 is reserved for schema initialization
-	`ALTER TABLE "Network" ALTER COLUMN nick DROP NOT NULL`,
-	`
-		CREATE TYPE sasl_mechanism AS ENUM ('PLAIN', 'EXTERNAL');
-		ALTER TABLE "Network"
-			ALTER COLUMN sasl_mechanism
-			TYPE sasl_mechanism
-			USING sasl_mechanism::sasl_mechanism;
-	`,
-	`
-		CREATE TABLE "ReadReceipt" (
-			id SERIAL PRIMARY KEY,
-			network INTEGER NOT NULL REFERENCES "Network"(id) ON DELETE CASCADE,
-			target VARCHAR(255) NOT NULL,
-			timestamp TIMESTAMP WITH TIME ZONE NOT NULL,
-			UNIQUE(network, target)
-		);
-	`,
-	`
-		CREATE TABLE "WebPushConfig" (
-			id SERIAL PRIMARY KEY,
-			created_at TIMESTAMP WITH TIME ZONE NOT NULL,
-			vapid_key_public TEXT NOT NULL,
-			vapid_key_private TEXT NOT NULL,
-			UNIQUE(vapid_key_public)
-		);
-
-		CREATE TABLE "WebPushSubscription" (
-			id SERIAL PRIMARY KEY,
-			created_at TIMESTAMP WITH TIME ZONE NOT NULL,
-			updated_at TIMESTAMP WITH TIME ZONE NOT NULL,
-			network INTEGER REFERENCES "Network"(id) ON DELETE CASCADE,
-			endpoint TEXT NOT NULL,
-			key_vapid TEXT,
-			key_auth TEXT,
-			key_p256dh TEXT,
-			UNIQUE(network, endpoint)
-		);
-	`,
-	`
-		ALTER TABLE "WebPushSubscription"
-		ADD COLUMN "user" INTEGER
-		REFERENCES "User"(id) ON DELETE CASCADE
-	`,
-	`ALTER TABLE "User" ADD COLUMN nick VARCHAR(255)`,
-	// Before this migration, a bug swapped user and network, so empty the
-	// web push subscriptions table
-	`
-		DELETE FROM "WebPushSubscription";
-		ALTER TABLE "WebPushSubscription"
-		ALTER COLUMN "user"
-		SET NOT NULL;
-	`,
-	`ALTER TABLE "Network" ADD COLUMN auto_away BOOLEAN NOT NULL DEFAULT TRUE`,
-	`ALTER TABLE "Network" ADD COLUMN certfp TEXT`,
-	`ALTER TABLE "User" ADD COLUMN created_at TIMESTAMP WITH TIME ZONE NOT NULL DEFAULT now()`,
-	`ALTER TABLE "User" ADD COLUMN enabled BOOLEAN NOT NULL DEFAULT TRUE`,
-	`ALTER TABLE "User" ADD COLUMN downstream_interacted_at TIMESTAMP WITH TIME ZONE`,
-	`
-		CREATE TABLE "MessageTarget" (
-			id SERIAL PRIMARY KEY,
-			network INTEGER NOT NULL REFERENCES "Network"(id) ON DELETE CASCADE,
-			target TEXT NOT NULL,
-			UNIQUE(network, target)
-		);
-		CREATE TEXT SEARCH DICTIONARY search_simple_dictionary (
-			TEMPLATE = pg_catalog.simple
-		);
-		CREATE TEXT SEARCH CONFIGURATION @SCHEMA_PREFIX@search_simple ( COPY = pg_catalog.simple );
-		ALTER TEXT SEARCH CONFIGURATION @SCHEMA_PREFIX@search_simple ALTER MAPPING FOR asciiword, asciihword, hword_asciipart, hword, hword_part, word WITH @SCHEMA_PREFIX@search_simple_dictionary;
-		CREATE TABLE "Message" (
-			id SERIAL PRIMARY KEY,
-			target INTEGER NOT NULL REFERENCES "MessageTarget"(id) ON DELETE CASCADE,
-			raw TEXT NOT NULL,
-			time TIMESTAMP WITH TIME ZONE NOT NULL,
-			sender TEXT NOT NULL,
-			text TEXT,
-			text_search tsvector GENERATED ALWAYS AS (to_tsvector('@SCHEMA_PREFIX@search_simple', text)) STORED
-		);
-		CREATE INDEX "MessageIndex" ON "Message" (target, time);
-		CREATE INDEX "MessageSearchIndex" ON "Message" USING GIN (text_search);
-	`,
-}
-
 type PostgresDB struct {
 	db   *sql.DB
 	temp bool
@@ -257,7 +173,7 @@ func (db *PostgresDB) ListUsers(ctx context.Context) ([]User, error) {
 
 	rows, err := db.db.QueryContext(ctx,
 		`SELECT id, username, password, admin, nick, realname, enabled,
-			downstream_interacted_at
+			downstream_interacted_at, max_networks
 		FROM "User"`)
 	if err != nil {
 		return nil, err
@@ -269,7 +185,7 @@ func (db *PostgresDB) ListUsers(ctx context.Context) ([]User, error) {
 		var user User
 		var password, nick, realname sql.NullString
 		var downstreamInteractedAt sql.NullTime
-		if err := rows.Scan(&user.ID, &user.Username, &password, &user.Admin, &nick, &realname, &user.Enabled, &downstreamInteractedAt); err != nil {
+		if err := rows.Scan(&user.ID, &user.Username, &password, &user.Admin, &nick, &realname, &user.Enabled, &downstreamInteractedAt, &user.MaxNetworks); err != nil {
 			return nil, err
 		}
 		user.Password = password.String
@@ -294,11 +210,11 @@ func (db *PostgresDB) GetUser(ctx context.Context, username string) (*User, erro
 	var password, nick, realname sql.NullString
 	var downstreamInteractedAt sql.NullTime
 	row := db.db.QueryRowContext(ctx,
-		`SELECT id, password, admin, nick, realname, enabled, downstream_interacted_at
+		`SELECT id, password, admin, nick, realname, enabled, downstream_interacted_at, max_networks
 		FROM "User"
 		WHERE username = $1`,
 		username)
-	if err := row.Scan(&user.ID, &password, &user.Admin, &nick, &realname, &user.Enabled, &downstreamInteractedAt); err != nil {
+	if err := row.Scan(&user.ID, &password, &user.Admin, &nick, &realname, &user.Enabled, &downstreamInteractedAt, &user.MaxNetworks); err != nil {
 		return nil, err
 	}
 	user.Password = password.String
@@ -348,19 +264,19 @@ func (db *PostgresDB) StoreUser(ctx context.Context, user *User) error {
 	if user.ID == 0 {
 		err = db.db.QueryRowContext(ctx, `
 			INSERT INTO "User" (username, password, admin, nick, realname,
-				enabled, downstream_interacted_at)
-			VALUES ($1, $2, $3, $4, $5, $6, $7)
+				enabled, downstream_interacted_at, max_networks)
+			VALUES ($1, $2, $3, $4, $5, $6, $7, $8)
 			RETURNING id`,
 			user.Username, password, user.Admin, nick, realname, user.Enabled,
-			downstreamInteractedAt).Scan(&user.ID)
+			downstreamInteractedAt, user.MaxNetworks).Scan(&user.ID)
 	} else {
 		_, err = db.db.ExecContext(ctx, `
 			UPDATE "User"
 			SET password = $1, admin = $2, nick = $3, realname = $4,
-				enabled = $5, downstream_interacted_at = $6
-			WHERE id = $7`,
+				enabled = $5, downstream_interacted_at = $6, max_networks = $7
+			WHERE id = $8`,
 			password, user.Admin, nick, realname, user.Enabled,
-			downstreamInteractedAt, user.ID)
+			downstreamInteractedAt, user.MaxNetworks, user.ID)
 	}
 	return err
 }
@@ -803,9 +719,12 @@ func (db *PostgresDB) GetMessageLastID(ctx context.Context, networkID int64, nam
 
 	var msgID int64
 	row := db.db.QueryRowContext(ctx, `
-		SELECT m.id FROM "Message" AS m, "MessageTarget" as t
-		WHERE t.network = $1 AND t.target = $2 AND m.target = t.id
-		ORDER BY m.time DESC LIMIT 1`,
+		SELECT id FROM "Message"
+		WHERE target = (
+			SELECT id FROM "MessageTarget"
+			WHERE network = $1 AND target = $2
+		)
+		ORDER BY time DESC LIMIT 1`,
 		networkID,
 		name,
 	)
@@ -818,6 +737,44 @@ func (db *PostgresDB) GetMessageLastID(ctx context.Context, networkID int64, nam
 	return msgID, nil
 }
 
+func (db *PostgresDB) GetMessageTarget(ctx context.Context, networkID int64, target string) (*MessageTarget, error) {
+	ctx, cancel := context.WithTimeout(ctx, postgresQueryTimeout)
+	defer cancel()
+
+	mt := &MessageTarget{
+		Target: target,
+	}
+
+	row := db.db.QueryRowContext(ctx,
+		`SELECT id, pinned, muted FROM "MessageTarget" WHERE network = $1 AND target = $2`,
+		networkID, target)
+	if err := row.Scan(&mt.ID, &mt.Pinned, &mt.Muted); err != nil && err != sql.ErrNoRows {
+		return nil, err
+	}
+	return mt, nil
+}
+
+func (db *PostgresDB) StoreMessageTarget(ctx context.Context, networkID int64, mt *MessageTarget) error {
+	ctx, cancel := context.WithTimeout(ctx, postgresQueryTimeout)
+	defer cancel()
+
+	var err error
+	if mt.ID != 0 {
+		_, err = db.db.ExecContext(ctx, `
+			UPDATE "MessageTarget"
+			SET pinned = $1, muted = $2
+			WHERE id = $3`,
+			mt.Pinned, mt.Muted, mt.ID)
+	} else {
+		err = db.db.QueryRowContext(ctx, `
+			INSERT INTO "MessageTarget" (network, target, pinned, muted)
+			VALUES ($1, $2, $3, $4)
+			RETURNING id`,
+			networkID, mt.Target, mt.Pinned, mt.Muted).Scan(&mt.ID)
+	}
+	return err
+}
+
 func (db *PostgresDB) StoreMessages(ctx context.Context, networkID int64, name string, msgs []*irc.Message) ([]int64, error) {
 	if len(msgs) == 0 {
 		return nil, nil
@@ -871,7 +828,7 @@ func (db *PostgresDB) StoreMessages(ctx context.Context, networkID int64, name s
 		case "PRIVMSG", "NOTICE":
 			if len(msg.Params) > 1 {
 				text.Valid = true
-				text.String = msg.Params[1]
+				text.String = stripANSI(msg.Params[1])
 			}
 		}
 
@@ -892,7 +849,7 @@ func (db *PostgresDB) StoreMessages(ctx context.Context, networkID int64, name s
 	return ids, err
 }
 
-func (db *PostgresDB) ListMessageLastPerTarget(ctx context.Context, networkID int64, options *MessageOptions) ([]MessageTarget, error) {
+func (db *PostgresDB) ListMessageLastPerTarget(ctx context.Context, networkID int64, options *MessageOptions) ([]MessageTargetLast, error) {
 	ctx, cancel := context.WithTimeout(ctx, postgresQueryTimeout)
 	defer cancel()
 
@@ -900,31 +857,38 @@ func (db *PostgresDB) ListMessageLastPerTarget(ctx context.Context, networkID in
 		networkID,
 	}
 	query := `
-		SELECT t.target, MAX(m.time) AS latest
-		FROM "Message" m, "MessageTarget" t
-		WHERE m.target = t.id AND t.network = $1
+		SELECT t.target, l.latest
+		FROM "MessageTarget" t JOIN LATERAL (
+			SELECT m.target, m.time AS latest, m.text
+			FROM "Message" m
+			WHERE m.target = t.id
 	`
+
 	if !options.Events {
 		query += `AND m.text IS NOT NULL `
 	}
+
 	query += `
-		GROUP BY t.target
-		HAVING true
+
+			ORDER BY m.time DESC LIMIT 1
+		) AS l ON t.id = l.target
+		WHERE t.network = $1
 	`
+
 	if !options.AfterTime.IsZero() {
 		// compares time strings by lexicographical order
 		parameters = append(parameters, options.AfterTime)
-		query += fmt.Sprintf(`AND MAX(m.time) > $%d `, len(parameters))
+		query += fmt.Sprintf(`AND l.latest > $%d `, len(parameters))
 	}
 	if !options.BeforeTime.IsZero() {
 		// compares time strings by lexicographical order
 		parameters = append(parameters, options.BeforeTime)
-		query += fmt.Sprintf(`AND MAX(m.time) < $%d `, len(parameters))
+		query += fmt.Sprintf(`AND l.latest < $%d `, len(parameters))
 	}
 	if options.TakeLast {
-		query += `ORDER BY latest DESC `
+		query += `ORDER BY l.latest DESC `
 	} else {
-		query += `ORDER BY latest ASC `
+		query += `ORDER BY l.latest ASC `
 	}
 	parameters = append(parameters, options.Limit)
 	query += fmt.Sprintf(`LIMIT $%d`, len(parameters))
@@ -935,9 +899,9 @@ func (db *PostgresDB) ListMessageLastPerTarget(ctx context.Context, networkID in
 	}
 	defer rows.Close()
 
-	var l []MessageTarget
+	var l []MessageTargetLast
 	for rows.Next() {
-		var mt MessageTarget
+		var mt MessageTargetLast
 		if err := rows.Scan(&mt.Name, &mt.LatestMessage); err != nil {
 			return nil, err
 		}
@@ -968,38 +932,40 @@ func (db *PostgresDB) ListMessages(ctx context.Context, networkID int64, name st
 		name,
 	}
 	query := `
-		SELECT m.raw
-		FROM "Message" AS m, "MessageTarget" AS t
-		WHERE m.target = t.id AND t.network = $1 AND t.target = $2 `
+		SELECT raw FROM "Message"
+		WHERE target = (
+			SELECT id FROM "MessageTarget"
+			WHERE network = $1 AND target = $2
+		) `
 	if options.AfterID > 0 {
 		parameters = append(parameters, options.AfterID)
-		query += fmt.Sprintf(`AND m.id > $%d `, len(parameters))
+		query += fmt.Sprintf(`AND id > $%d `, len(parameters))
 	}
 	if !options.AfterTime.IsZero() {
 		// compares time strings by lexicographical order
 		parameters = append(parameters, options.AfterTime)
-		query += fmt.Sprintf(`AND m.time > $%d `, len(parameters))
+		query += fmt.Sprintf(`AND time > $%d `, len(parameters))
 	}
 	if !options.BeforeTime.IsZero() {
 		// compares time strings by lexicographical order
 		parameters = append(parameters, options.BeforeTime)
-		query += fmt.Sprintf(`AND m.time < $%d `, len(parameters))
+		query += fmt.Sprintf(`AND time < $%d `, len(parameters))
 	}
 	if options.Sender != "" {
 		parameters = append(parameters, options.Sender)
-		query += fmt.Sprintf(`AND m.sender = $%d `, len(parameters))
+		query += fmt.Sprintf(`AND sender = $%d `, len(parameters))
 	}
 	if options.Text != "" {
 		parameters = append(parameters, options.Text)
 		query += fmt.Sprintf(`AND text_search @@ plainto_tsquery('search_simple', $%d) `, len(parameters))
 	}
 	if !options.Events {
-		query += `AND m.text IS NOT NULL `
+		query += `AND text IS NOT NULL `
 	}
 	if options.TakeLast {
-		query += `ORDER BY m.time DESC `
+		query += `ORDER BY time DESC `
 	} else {
-		query += `ORDER BY m.time ASC `
+		query += `ORDER BY time ASC `
 	}
 	parameters = append(parameters, options.Limit)
 	query += fmt.Sprintf(`LIMIT $%d`, len(parameters))
