@@ -1,8 +1,10 @@
 package fileupload
 
 import (
+	"context"
 	"crypto/rand"
 	"encoding/hex"
+	"errors"
 	"fmt"
 	"io"
 	"mime"
@@ -23,20 +25,21 @@ const maxSize = 50 * 1024 * 1024 // 50 MiB
 // browser to execute any kind of script. For instance, SVG images are
 // intentionally omitted.
 var inlineMIMETypes = map[string]bool{
-	"audio/aac":  true,
-	"audio/mp4":  true,
-	"audio/mpeg": true,
-	"audio/ogg":  true,
-	"audio/webm": true,
-	"image/apng": true,
-	"image/gif":  true,
-	"image/jpeg": true,
-	"image/png":  true,
-	"image/webp": true,
-	"text/plain": true,
-	"video/mp4":  true,
-	"video/ogg":  true,
-	"video/webm": true,
+	"application/pdf": true,
+	"audio/aac":       true,
+	"audio/mp4":       true,
+	"audio/mpeg":      true,
+	"audio/ogg":       true,
+	"audio/webm":      true,
+	"image/apng":      true,
+	"image/gif":       true,
+	"image/jpeg":      true,
+	"image/png":       true,
+	"image/webp":      true,
+	"text/plain":      true,
+	"video/mp4":       true,
+	"video/ogg":       true,
+	"video/webm":      true,
 }
 
 // Some MIME types have multiple possible extensions, and
@@ -56,15 +59,40 @@ var primaryExts = map[string]string{
 	"video/mp4":  "mp4",
 }
 
+type httpError struct {
+	Code        int
+	ContentType string
+	Body        io.Reader
+}
+
+func (h *httpError) Error() string {
+	return fmt.Sprintf("error %v: %v", h.Code, h.Body)
+}
+
+func (h *httpError) Write(w http.ResponseWriter) {
+	defer func() {
+		if c, ok := h.Body.(io.Closer); ok {
+			c.Close()
+		}
+	}()
+	if h.ContentType != "" {
+		w.Header().Set("Content-Type", h.ContentType)
+	}
+	w.WriteHeader(h.Code)
+	io.Copy(w, h.Body)
+}
+
 type Uploader interface {
-	load(filename string) (basename string, modTime time.Time, content io.ReadSeekCloser, err error)
-	store(r io.Reader, username, mimeType, basename string) (outFilename string, err error)
+	load(ctx context.Context, filename string) (basename string, modTime time.Time, content io.ReadSeekCloser, err error)
+	store(ctx context.Context, r io.Reader, username, mimeType, basename string) (out string, err error)
 }
 
 func New(driver, source string) (Uploader, error) {
 	switch driver {
 	case "fs":
-		return &fs{source}, nil
+		return &fileuploadFS{source}, nil
+	case "http":
+		return &fileuploadHTTP{source}, nil
 	default:
 		return nil, fmt.Errorf("unknown file upload driver %q", driver)
 	}
@@ -72,7 +100,7 @@ func New(driver, source string) (Uploader, error) {
 
 type Handler struct {
 	Uploader    Uploader
-	Auth        auth.Authenticator
+	Auth        *auth.Authenticator
 	DB          database.Database
 	HTTPOrigins []string
 }
@@ -113,7 +141,7 @@ func (h *Handler) setCORS(resp http.ResponseWriter, req *http.Request) error {
 }
 
 func (h *Handler) ServeHTTP(resp http.ResponseWriter, req *http.Request) {
-	resp.Header().Set("Content-Security-Policy", "sandbox; default-src 'none'; script-src 'none';")
+	resp.Header().Set("Content-Security-Policy", "sandbox; default-src 'none'; script-src 'none'; media-src 'self';")
 
 	if err := h.setCORS(resp, req); err != nil {
 		http.Error(resp, err.Error(), http.StatusForbidden)
@@ -151,8 +179,13 @@ func (h *Handler) fetch(resp http.ResponseWriter, req *http.Request) {
 		return
 	}
 
-	basename, modTime, content, err := h.Uploader.load(filename)
+	basename, modTime, content, err := h.Uploader.load(req.Context(), filename)
 	if err != nil {
+		var httpErr *httpError
+		if errors.As(err, &httpErr) {
+			httpErr.Write(resp)
+			return
+		}
 		http.Error(resp, "failed to open file", http.StatusNotFound)
 		return
 	}
@@ -207,12 +240,13 @@ func (h *Handler) store(resp http.ResponseWriter, req *http.Request) {
 	scheme, param, _ := strings.Cut(authz, " ")
 	switch strings.ToLower(scheme) {
 	case "basic":
-		plainAuth, ok := h.Auth.(auth.PlainAuthenticator)
-		if !ok {
+		plainAuth := h.Auth.Plain
+		if plainAuth == nil {
 			http.Error(resp, "Basic scheme in Authorization header not supported", http.StatusBadRequest)
 			return
 		}
 		var password string
+		var ok bool
 		username, password, ok = req.BasicAuth()
 		if !ok {
 			http.Error(resp, "invalid Authorization header", http.StatusBadRequest)
@@ -220,8 +254,8 @@ func (h *Handler) store(resp http.ResponseWriter, req *http.Request) {
 		}
 		err = plainAuth.AuthPlain(req.Context(), h.DB, username, password)
 	case "bearer":
-		oauthAuth, ok := h.Auth.(auth.OAuthBearerAuthenticator)
-		if !ok {
+		oauthAuth := h.Auth.OAuthBearer
+		if oauthAuth == nil {
 			http.Error(resp, "Bearer scheme in Authorization header not supported", http.StatusBadRequest)
 			return
 		}
@@ -275,34 +309,47 @@ func (h *Handler) store(resp http.ResponseWriter, req *http.Request) {
 		basename = path.Base(params["filename"])
 	}
 
-	r := &limitedReader{r: req.Body, n: maxSize}
-	outFilename, err := h.Uploader.store(r, username, mimeType, basename)
+	var out string
+	if req.ContentLength > maxSize {
+		// Check the Content-Length, best-effort, before we start reading from it.
+		// If the client sends us the length right away, we can signal to him immediately
+		// that the file is too large, before it uploads the entire file.
+		err = &http.MaxBytesError{
+			Limit: maxSize,
+		}
+	} else {
+		r := http.MaxBytesReader(resp, req.Body, maxSize)
+		out, err = h.Uploader.store(req.Context(), r, username, mimeType, basename)
+	}
+	if err != nil {
+		var httpErr *httpError
+		if errors.As(err, &httpErr) {
+			httpErr.Write(resp)
+			return
+		}
+		status := http.StatusInternalServerError
+		var maxBytesErr *http.MaxBytesError
+		if errors.As(err, &maxBytesErr) {
+			status = http.StatusRequestEntityTooLarge
+			resp.Header().Set("Connection", "close")
+			resp.Header().Set("Upload-Limit", fmt.Sprintf("maxsize=%v", maxBytesErr.Limit))
+		}
+		http.Error(resp, "failed to write file", status)
+		return
+	}
+
+	u, err := url.Parse(out)
 	if err != nil {
 		http.Error(resp, "failed to write file", http.StatusInternalServerError)
 		return
 	}
+	baseURL := url.URL{Path: "/uploads/"}
+	out = baseURL.ResolveReference(u).String()
 
-	resp.Header().Set("Location", "/uploads/"+outFilename)
+	resp.Header().Set("Location", out)
 	resp.WriteHeader(http.StatusCreated)
 }
 
-type limitedReader struct {
-	r io.Reader
-	n int64
-}
-
-func (lr *limitedReader) Read(p []byte) (n int, err error) {
-	if lr.n <= 0 {
-		return 0, fmt.Errorf("file too large")
-	}
-	if int64(len(p)) > lr.n {
-		p = p[0:lr.n]
-	}
-	n, err = lr.r.Read(p)
-	lr.n -= int64(n)
-	return n, err
-}
-
 func generateToken(n int) (string, error) {
 	b := make([]byte, n)
 	if _, err := rand.Read(b); err != nil {
