@@ -163,7 +163,12 @@ func handleServiceCommand(ctx *serviceContext, words []string) error {
 		return fmt.Errorf("command %q not found", words[0])
 	}
 
-	return cmd.handle(ctx, params)
+	if err := cmd.handle(ctx, params); err == flag.ErrHelp {
+		name := strings.Join(words[:len(words)-len(params)], " ")
+		return fmt.Errorf(`unsupported flag (type "help %v" for a help message)`, name)
+	} else {
+		return err
+	}
 }
 
 func (cmds serviceCommandSet) Get(params []string) (*serviceCommand, []string, error) {
@@ -218,7 +223,7 @@ func init() {
 		"network": {
 			children: serviceCommandSet{
 				"create": {
-					usage:  "-addr <addr> [-name name] [-username username] [-pass pass] [-realname realname] [-certfp fingerprint] [-nick nick] [-auto-away auto-away] [-enabled enabled] [-connect-command command]...",
+					usage:  "-addr <addr> [-name name] [-username username] [-pass pass] [-realname realname] [-certfp fingerprint] [-nick nick] [-auto-away auto-away] [-enabled enabled] [-ignore-limit ignore-limit] [-connect-command command]...",
 					desc:   "add a new network",
 					handle: handleServiceNetworkCreate,
 				},
@@ -227,7 +232,7 @@ func init() {
 					handle: handleServiceNetworkStatus,
 				},
 				"update": {
-					usage:  "[name] [-addr addr] [-name name] [-username username] [-pass pass] [-realname realname] [-certfp fingerprint] [-nick nick] [-auto-away auto-away] [-enabled enabled] [-connect-command command]...",
+					usage:  "[name] [-addr addr] [-name name] [-username username] [-pass pass] [-realname realname] [-certfp fingerprint] [-nick nick] [-auto-away auto-away] [-enabled enabled] [-ignore-limit ignore-limit] [-connect-command command]...",
 					desc:   "update a network",
 					handle: handleServiceNetworkUpdate,
 				},
@@ -286,14 +291,14 @@ func init() {
 					global: true,
 				},
 				"create": {
-					usage:  "-username <username> -password <password> [-disable-password] [-admin true|false] [-nick <nick>] [-realname <realname>] [-enabled true|false]",
+					usage:  "-username <username> -password <password> [-disable-password] [-admin true|false] [-nick <nick>] [-realname <realname>] [-enabled true|false]  [-max-networks <max-networks>]",
 					desc:   "create a new soju user",
 					handle: handleUserCreate,
 					admin:  true,
 					global: true,
 				},
 				"update": {
-					usage:  "[username] [-password <password>] [-disable-password] [-admin true|false] [-nick <nick>] [-realname <realname>] [-enabled true|false]",
+					usage:  "[username] [-password <password>] [-disable-password] [-admin true|false] [-nick <nick>] [-realname <realname>] [-enabled true|false] [-max-networks <max-networks>]",
 					desc:   "update a user",
 					handle: handleUserUpdate,
 					global: true,
@@ -321,8 +326,13 @@ func init() {
 					desc:   "show a list of saved channels and their current status",
 					handle: handleServiceChannelStatus,
 				},
+				"create": {
+					usage:  "<name> [-detached true|false] [-relay-detached default|none|highlight|message] [-reattach-on default|none|highlight|message] [-detach-after <duration>] [-detach-on default|none|highlight|message]",
+					desc:   "create a channel",
+					handle: handleServiceChannelCreate,
+				},
 				"update": {
-					usage:  "<name> [-detached <true|false>] [-relay-detached <default|none|highlight|message>] [-reattach-on <default|none|highlight|message>] [-detach-after <duration>] [-detach-on <default|none|highlight|message>]",
+					usage:  "<name> [-detached true|false] [-relay-detached default|none|highlight|message] [-reattach-on default|none|highlight|message] [-detach-after <duration>] [-detach-on default|none|highlight|message]",
 					desc:   "update a channel",
 					handle: handleServiceChannelUpdate,
 				},
@@ -348,6 +358,13 @@ func init() {
 					admin:  true,
 					global: true,
 				},
+				"debug": {
+					usage:  "true|false",
+					desc:   "enable/disable debug logging to stderr (will leak sensitive information)",
+					handle: handleServiceServerDebug,
+					admin:  true,
+					global: true,
+				},
 			},
 			admin: true,
 		},
@@ -458,6 +475,26 @@ func (f boolPtrFlag) Set(s string) error {
 	return nil
 }
 
+type intPtrFlag struct {
+	ptr **int
+}
+
+func (f intPtrFlag) String() string {
+	if f.ptr == nil || *f.ptr == nil {
+		return "<nil>"
+	}
+	return strconv.Itoa(**f.ptr)
+}
+
+func (f intPtrFlag) Set(s string) error {
+	v, err := strconv.Atoi(s)
+	if err != nil {
+		return err
+	}
+	*f.ptr = &v
+	return nil
+}
+
 func getNetworkFromArg(ctx *serviceContext, params []string) (*network, []string, error) {
 	name, params := popArg(params)
 	if name == "" {
@@ -478,6 +515,7 @@ type networkFlagSet struct {
 	*flag.FlagSet
 	Addr, Name, Nick, Username, Pass, Realname, CertFP *string
 	AutoAway, Enabled                                  *bool
+	IgnoreLimit                                        bool
 	ConnectCommands                                    []string
 }
 
@@ -492,6 +530,7 @@ func newNetworkFlagSet() *networkFlagSet {
 	fs.Var(stringPtrFlag{&fs.CertFP}, "certfp", "")
 	fs.Var(boolPtrFlag{&fs.AutoAway}, "auto-away", "")
 	fs.Var(boolPtrFlag{&fs.Enabled}, "enabled", "")
+	fs.BoolVar(&fs.IgnoreLimit, "ignore-limit", false, "")
 	fs.Var((*stringSliceFlag)(&fs.ConnectCommands), "connect-command", "")
 	return fs
 }
@@ -510,7 +549,7 @@ func (fs *networkFlagSet) update(network *database.Network) error {
 	}
 	if fs.Name != nil {
 		if *fs.Name == "*" {
-			return fmt.Errorf("the network name %q is reserved for multi-upstream mode", *fs.Name)
+			return fmt.Errorf("the network name %q is reserved", *fs.Name)
 		}
 		network.Name = *fs.Name
 	}
@@ -577,13 +616,16 @@ func handleServiceNetworkCreate(ctx *serviceContext, params []string) error {
 	if fs.Addr == nil {
 		return fmt.Errorf("flag -addr is required")
 	}
+	if fs.IgnoreLimit && !ctx.admin {
+		return fmt.Errorf("you must be an admin to use the flag -ignore-limit")
+	}
 
 	record := database.NewNetwork(*fs.Addr)
 	if err := fs.update(record); err != nil {
 		return err
 	}
 
-	network, err := ctx.user.createNetwork(ctx, record)
+	network, err := ctx.user.createNetwork(ctx, record, !fs.IgnoreLimit)
 	if err != nil {
 		return fmt.Errorf("could not create network: %v", err)
 	}
@@ -655,13 +697,17 @@ func handleServiceNetworkUpdate(ctx *serviceContext, params []string) error {
 	if fs.NArg() > 0 {
 		return fmt.Errorf("unexpected argument: %v", fs.Arg(0))
 	}
+	if fs.IgnoreLimit && !ctx.admin {
+		return fmt.Errorf("you must be an admin to use the flag -ignore-limit")
+	}
 
 	record := net.Network // copy network record because we'll mutate it
+	wasEnabled := record.Enabled
 	if err := fs.update(&record); err != nil {
 		return err
 	}
 
-	network, err := ctx.user.updateNetwork(ctx, &record)
+	network, err := ctx.user.updateNetwork(ctx, &record, !fs.IgnoreLimit && !wasEnabled)
 	if err != nil {
 		return fmt.Errorf("could not update network: %v", err)
 	}
@@ -948,6 +994,9 @@ func handleUserStatus(ctx *serviceContext, params []string) error {
 			return fmt.Errorf("could not get networks of user %q: %v", user.Username, err)
 		}
 		line += fmt.Sprintf(": %d networks", len(networks))
+		if user.MaxNetworks >= 0 {
+			line += fmt.Sprintf(" (%d max)", user.MaxNetworks)
+		}
 		ctx.print(line)
 	}
 	if n > len(users) {
@@ -966,6 +1015,7 @@ func handleUserCreate(ctx *serviceContext, params []string) error {
 	realname := fs.String("realname", "", "")
 	admin := fs.Bool("admin", false, "")
 	enabled := fs.Bool("enabled", true, "")
+	maxNetworks := fs.Int("max-networks", -1, "")
 
 	if err := fs.Parse(params); err != nil {
 		return err
@@ -988,6 +1038,7 @@ func handleUserCreate(ctx *serviceContext, params []string) error {
 	user.Realname = *realname
 	user.Admin = *admin
 	user.Enabled = *enabled
+	user.MaxNetworks = *maxNetworks
 	if !*disablePassword {
 		if err := user.SetPassword(*password); err != nil {
 			return err
@@ -1012,6 +1063,7 @@ func handleUserUpdate(ctx *serviceContext, params []string) error {
 	var password, nick, realname *string
 	var admin, enabled *bool
 	var disablePassword bool
+	var maxNetworks *int
 	fs := newFlagSet()
 	fs.Var(stringPtrFlag{&password}, "password", "")
 	fs.BoolVar(&disablePassword, "disable-password", false, "")
@@ -1019,6 +1071,7 @@ func handleUserUpdate(ctx *serviceContext, params []string) error {
 	fs.Var(stringPtrFlag{&realname}, "realname", "")
 	fs.Var(boolPtrFlag{&admin}, "admin", "")
 	fs.Var(boolPtrFlag{&enabled}, "enabled", "")
+	fs.Var(intPtrFlag{&maxNetworks}, "max-networks", "")
 
 	username, params := popArg(params)
 	if err := fs.Parse(params); err != nil {
@@ -1066,10 +1119,11 @@ func handleUserUpdate(ctx *serviceContext, params []string) error {
 
 		done := make(chan error, 1)
 		event := eventUserUpdate{
-			password: hashed,
-			admin:    admin,
-			enabled:  enabled,
-			done:     done,
+			password:    hashed,
+			admin:       admin,
+			enabled:     enabled,
+			maxNetworks: maxNetworks,
+			done:        done,
 		}
 		select {
 		case <-ctx.Done():
@@ -1089,6 +1143,9 @@ func handleUserUpdate(ctx *serviceContext, params []string) error {
 		if enabled != nil {
 			return fmt.Errorf("cannot update -enabled of own user")
 		}
+		if maxNetworks != nil && !ctx.admin {
+			return fmt.Errorf("cannot update -max-networks of own user")
+		}
 
 		err := ctx.user.updateUser(ctx, func(record *database.User) error {
 			if password != nil {
@@ -1105,6 +1162,9 @@ func handleUserUpdate(ctx *serviceContext, params []string) error {
 			if realname != nil {
 				record.Realname = *realname
 			}
+			if maxNetworks != nil {
+				record.MaxNetworks = *maxNetworks
+			}
 			return nil
 		})
 		if err != nil {
@@ -1169,6 +1229,11 @@ func handleUserDelete(ctx *serviceContext, params []string) error {
 	return nil
 }
 
+type userRunMsg struct {
+	message string
+	err     error
+}
+
 func handleUserRun(ctx *serviceContext, params []string) error {
 	if len(params) < 2 {
 		return fmt.Errorf("expected at least two arguments")
@@ -1185,12 +1250,10 @@ func handleUserRun(ctx *serviceContext, params []string) error {
 		return fmt.Errorf("unknown username %q", username)
 	}
 
-	printCh := make(chan string, 1)
-	retCh := make(chan error, 1)
+	msgCh := make(chan userRunMsg, 1)
 	ev := eventUserRun{
 		params: params,
-		print:  printCh,
-		ret:    retCh,
+		ch:     msgCh,
 	}
 	select {
 	case <-ctx.Done():
@@ -1209,12 +1272,12 @@ func handleUserRun(ctx *serviceContext, params []string) error {
 			// TODO: Properly fix this condition by flushing the u.events queue
 			//       and running close(ev.print) in a defer
 			return fmt.Errorf("timeout executing command")
-		case text, ok := <-printCh:
-			if ok {
-				ctx.print(text)
+		case msg := <-msgCh:
+			if msg.message != "" {
+				ctx.print(msg.message)
+			} else {
+				return msg.err
 			}
-		case ret := <-retCh:
-			return ret
 		}
 	}
 }
@@ -1381,6 +1444,58 @@ func stripNetworkSuffix(ctx *serviceContext, name string) (string, *network, err
 	return "", nil, fmt.Errorf("unknown network %q", netName)
 }
 
+func handleServiceChannelCreate(ctx *serviceContext, params []string) error {
+	if len(params) < 1 {
+		return fmt.Errorf("expected at least one argument")
+	}
+	name := params[0]
+
+	fs := newChannelFlagSet()
+	if err := fs.Parse(params[1:]); err != nil {
+		return err
+	}
+	if fs.NArg() > 0 {
+		return fmt.Errorf("unexpected argument: %v", fs.Arg(0))
+	}
+
+	name, network, err := stripNetworkSuffix(ctx, name)
+	if err != nil {
+		return err
+	}
+	if name == "" || strings.ContainsAny(name, illegalChanChars) {
+		return fmt.Errorf("invalid channel name: %v", name)
+	}
+	if network.conn != nil && !network.conn.isChannel(name) {
+		return fmt.Errorf("not a channel name: %v", name)
+	}
+	if network.channels.Get(name) != nil {
+		return fmt.Errorf("channel %q already exists", name)
+	}
+
+	ch := database.Channel{
+		Name: name,
+	}
+	if err := fs.update(&ch); err != nil {
+		return err
+	}
+
+	network.channels.Set(ch.Name, &ch)
+
+	if err := ctx.srv.db.StoreChannel(ctx, network.ID, &ch); err != nil {
+		return fmt.Errorf("failed to create channel: %v", err)
+	}
+
+	if network.conn != nil {
+		network.conn.SendMessage(ctx, &irc.Message{
+			Command: "JOIN",
+			Params:  []string{ch.Name},
+		})
+	}
+
+	ctx.print(fmt.Sprintf("created channel %q", name))
+	return nil
+}
+
 func handleServiceChannelUpdate(ctx *serviceContext, params []string) error {
 	if len(params) < 1 {
 		return fmt.Errorf("expected at least one argument")
@@ -1506,3 +1621,24 @@ func handleServiceServerNotice(ctx *serviceContext, params []string) error {
 
 	return err
 }
+
+func handleServiceServerDebug(ctx *serviceContext, params []string) error {
+	if len(params) != 1 {
+		return fmt.Errorf("expected exactly one argument")
+	}
+
+	enabled, err := strconv.ParseBool(params[0])
+	if err != nil {
+		return err
+	}
+
+	previous := ctx.srv.Logger.debug.Swap(enabled)
+	if previous != enabled {
+		if enabled {
+			ctx.srv.Logger.Printf("enabling debug logging")
+		} else {
+			ctx.srv.Logger.Printf("disabling debug logging")
+		}
+	}
+	return nil
+}
