@@ -75,10 +75,11 @@ type eventBroadcast struct {
 type eventStop struct{}
 
 type eventUserUpdate struct {
-	password *string
-	admin    *bool
-	enabled  *bool
-	done     chan error
+	password    *string
+	admin       *bool
+	enabled     *bool
+	maxNetworks *int
+	done        chan error
 }
 
 type eventTryRegainNick struct {
@@ -88,8 +89,7 @@ type eventTryRegainNick struct {
 
 type eventUserRun struct {
 	params []string
-	print  chan string
-	ret    chan error
+	ch     chan userRunMsg
 }
 
 type deliveredClientMap map[string]string // client name -> msg ID
@@ -102,6 +102,10 @@ func newDeliveredStore(cm xirc.CaseMapping) deliveredStore {
 	return deliveredStore{xirc.NewCaseMappingMap[deliveredClientMap](cm)}
 }
 
+func (ds deliveredStore) Empty() bool {
+	return ds.m.Len() == 0
+}
+
 func (ds deliveredStore) HasTarget(target string) bool {
 	return ds.m.Get(target) != nil
 }
@@ -218,9 +222,11 @@ func (net *network) runConn(ctx context.Context) error {
 	net.user.srv.metrics.upstreams.Add(1)
 	defer net.user.srv.metrics.upstreams.Add(-1)
 
-	done := ctx.Done()
-	ctx, cancel := context.WithTimeout(ctx, time.Minute)
+	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
+	done := ctx.Done() // This must not be subject to timeout
+	ctx, cancelTimeout := context.WithTimeout(ctx, time.Minute)
+	defer cancelTimeout()
 
 	uc, err := connectToUpstream(ctx, net)
 	if err != nil {
@@ -767,7 +773,9 @@ func (u *user) run() {
 				uc.updateAway()
 			})
 
-			u.bumpDownstreamInteractionTime(ctx)
+			if !dc.impersonating {
+				u.bumpDownstreamInteractionTime(ctx)
+			}
 		case eventDownstreamDisconnected:
 			dc := e.dc
 			ctx := context.TODO()
@@ -790,7 +798,9 @@ func (u *user) run() {
 				uc.updateMonitor()
 			})
 
-			u.bumpDownstreamInteractionTime(ctx)
+			if !dc.impersonating {
+				u.bumpDownstreamInteractionTime(ctx)
+			}
 		case eventDownstreamMessage:
 			msg, dc := e.msg, e.dc
 			if dc.isClosed() {
@@ -821,6 +831,9 @@ func (u *user) run() {
 				if e.enabled != nil {
 					record.Enabled = *e.enabled
 				}
+				if e.maxNetworks != nil {
+					record.MaxNetworks = *e.maxNetworks
+				}
 				return nil
 			})
 
@@ -839,20 +852,27 @@ func (u *user) run() {
 				Context: ctx,
 				user:    u,
 				srv:     u.srv,
-				admin:   u.Admin,
+				// Here we are setting an admin context on any user run command
+				// that is run.
+				// As a reminder, user run can only be run by an admin.
+				// This enables admins to run actions on a user with admin rights,
+				// for example to add a network past the user limit.
+				// Non-admin users cannot run user run, not even on themselves,
+				// so this cannot be used to escalate privileges.
+				admin: true,
 				print: func(text string) {
 					// Avoid blocking on e.print in case our context is canceled.
 					// This is a no-op right now because we use context.TODO(),
 					// but might be useful later when we add timeouts.
 					select {
 					case <-ctx.Done():
-					case e.print <- text:
+					case e.ch <- userRunMsg{message: text}:
 					}
 				},
 			}, e.params)
 			select {
 			case <-ctx.Done():
-			case e.ret <- err:
+			case e.ch <- userRunMsg{err: err}:
 			}
 		case eventStop:
 			for _, dc := range u.downstreamConns {
@@ -962,6 +982,26 @@ func (u *user) removeNetwork(network *network) {
 	panic("tried to remove a non-existing network")
 }
 
+func (u *user) canEnableNewNetwork() error {
+	max := u.MaxNetworks
+	if max < 0 {
+		max = u.srv.Config().MaxUserNetworks
+	}
+	if max < 0 {
+		return nil
+	}
+	n := 0
+	for _, network := range u.networks {
+		if network.Enabled {
+			n++
+		}
+	}
+	if n >= max {
+		return fmt.Errorf("maximum number of enabled networks reached")
+	}
+	return nil
+}
+
 func (u *user) checkNetwork(record *database.Network) error {
 	url, err := record.URL()
 	if err != nil {
@@ -985,6 +1025,9 @@ func (u *user) checkNetwork(record *database.Network) error {
 			return fmt.Errorf("%v:// URL must not have a path", url.Scheme)
 		}
 	case "irc+unix", "unix":
+		if !u.Admin {
+			return fmt.Errorf("non-admin users cannot add networks with a unix address")
+		}
 		if url.Host != "" {
 			return fmt.Errorf("%v:// URL must not have a host", url.Scheme)
 		}
@@ -1012,17 +1055,19 @@ func (u *user) checkNetwork(record *database.Network) error {
 	return nil
 }
 
-func (u *user) createNetwork(ctx context.Context, record *database.Network) (*network, error) {
+func (u *user) createNetwork(ctx context.Context, record *database.Network, enforceLimit bool) (*network, error) {
 	if record.ID != 0 {
 		panic("tried creating an already-existing network")
 	}
 
-	if err := u.checkNetwork(record); err != nil {
-		return nil, err
+	if enforceLimit && record.Enabled {
+		if err := u.canEnableNewNetwork(); err != nil {
+			return nil, err
+		}
 	}
 
-	if max := u.srv.Config().MaxUserNetworks; max >= 0 && len(u.networks) >= max {
-		return nil, fmt.Errorf("maximum number of networks reached")
+	if err := u.checkNetwork(record); err != nil {
+		return nil, err
 	}
 
 	network := newNetwork(u, record, nil)
@@ -1039,11 +1084,17 @@ func (u *user) createNetwork(ctx context.Context, record *database.Network) (*ne
 	return network, nil
 }
 
-func (u *user) updateNetwork(ctx context.Context, record *database.Network) (*network, error) {
+func (u *user) updateNetwork(ctx context.Context, record *database.Network, enforceLimit bool) (*network, error) {
 	if record.ID == 0 {
 		panic("tried updating a new network")
 	}
 
+	if enforceLimit && record.Enabled {
+		if err := u.canEnableNewNetwork(); err != nil {
+			return nil, err
+		}
+	}
+
 	// If the nickname/realname is reset to the default, just wipe the
 	// per-network setting
 	if record.Nick == u.Nick {
@@ -1196,7 +1247,7 @@ func (u *user) updateUser(ctx context.Context, update UserUpdateFunc) error {
 
 		var netErr error
 		for _, net := range needUpdate {
-			if _, err := u.updateNetwork(ctx, &net); err != nil {
+			if _, err := u.updateNetwork(ctx, &net, false); err != nil {
 				netErr = err
 			}
 		}
